#include "http_loop.h"
#include "http.h"

http_loop_gctx *http_loop_gctx_init() {
  http_loop_gctx *gctx = calloc(sizeof(http_loop_gctx), 1);

  return gctx;
}

http_loop_ctx *http_loop_ctx_init(http_loop_gctx *g) {
  http_loop_ctx *ctx = calloc(sizeof(http_loop_ctx), 1);
  ctx->g = g;

  return ctx;
}

void http_loop_ctx_free(http_loop_ctx *ctx) { free(ctx); }

bool http_loop_handle_request(event_loop_conn *conn) {
  // Prevents the request handler function from looping indefinitely without
  // ever consuming new data
  if (conn->rbuf_size - conn->rbuf_read == 0) {
    return false;
  }

  http_loop_ctx *ctx = conn->ctx;

  // If route is defined, we're currently processing a request
  if (ctx->route == NULL) {
    http_parse_error res = http_loop_parse_request(conn);

    if (res == http_parse_error_invalid ||
        (res == http_parse_error_incomplete &&
         conn->rbuf_size == EVENT_LOOP_BUFFER_SIZE)) {
      conn->state = event_loop_conn_state_end;

      return false;
    }
    // It's fun to respond with extremely specific error messages
    // https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/501
    else if (res == http_parse_error_unknown_method) {
      http_loop_write_standard_response(conn, 501);
      return false;
    }

    conn->rbuf_read += res;

    // If the routing fails, we exit the handler
    if (!http_loop_route_request(conn)) {
      return false;
    }
  }

  http_loop_process_request(conn);

  // TODO in highly concurrent situations, it might actually be better to always
  // return false here, as this allows cycling better through all connections
  return conn->state == event_loop_conn_state_req;
}

event_loop *http_loop_init(http_loop_gctx *gctx) {
  event_loop *el = event_loop_init();

  el->ctx_init = (void *(*)(void *))http_loop_ctx_init;
  el->ctx_free = (void (*)(void *))http_loop_ctx_free;
  el->handle_data = http_loop_handle_request;
  el->gctx = gctx;

  return el;
}
