#include <stdio.h>

#include "http.h"
#include "http_loop.h"
#include "log.h"

bool http_write_404(event_loop_conn *conn) {
  memcpy(conn->wbuf, http_405, http_405_len);

  conn->state = event_loop_conn_state_res;
  conn->wbuf_size = http_405_len;
  conn->wbuf_sent = 0;

  return true;
}

http_route routes[] = {
    {.type = http_route_literal, .path = "/", .steps = {http_write_404, NULL}}};

int main() {
  setvbuf(stdout, NULL, _IONBF, 0);

  info("Initializing trie");

  Trie *trie = trie_init();
  int count = trie_populate(trie, "lander.data");

  if (count < 0) {
    critical(1, "An error occured while populating the trie.");
  }

  info("Trie initialized and populated with %i entries", count);

  http_loop_gctx *gctx = http_loop_gctx_init();
  gctx->trie = trie;
  gctx->routes = routes;
  gctx->route_count = sizeof(routes) / sizeof(routes[0]);
  event_loop *el = http_loop_init(gctx);

  event_loop_run(el, 8000);
}
